@@ -63,6 +63,8 @@ struct _GgvPostScriptViewPrivate {
 	BonoboControl *owner;
 
 	GdkCursor *pan_cursor, *win_cursor;
+
+	GList *dialogs;
 };
 
 struct _GgvPostScriptViewClassPrivate {
@@ -120,18 +122,6 @@ static const gchar *size_paths[] = {
 	NULL
 };
 
-void
-get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
-{
-	const gchar *status;
-
-	g_assert(arg->_type == TC_CORBA_string);
-	
-	status = GTK_GS(ps_view->priv->gs)->gs_status;;
-
-	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
-}
-
 static void
 get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 {
@@ -150,6 +140,47 @@ get_title(GgvPostScriptView *ps_view, BonoboArg *arg)
 	*(CORBA_string *)arg->_value = CORBA_string_dup(title);
 }
 
+static void
+get_status(GgvPostScriptView *ps_view, BonoboArg *arg)
+{
+	const gchar *status;
+
+	g_assert(arg->_type == TC_CORBA_string);
+	
+	status = GTK_GS(ps_view->priv->gs)->gs_status;;
+
+	*(CORBA_string *)arg->_value = CORBA_string_dup(status);
+}
+
+static void
+remove_dialog_from_list(GtkWidget *w, gpointer user_data)
+{
+	GgvPostScriptView *ps_view = GGV_POSTSCRIPT_VIEW(user_data);
+
+	ps_view->priv->dialogs = g_list_remove(ps_view->priv->dialogs, w);
+}
+
+static void
+add_dialog_to_view(GgvPostScriptView *ps_view, GtkWidget *dlg)
+{
+	ps_view->priv->dialogs = g_list_prepend(ps_view->priv->dialogs, dlg);
+	g_signal_connect(G_OBJECT(dlg), "destroy",
+					 G_CALLBACK(remove_dialog_from_list), ps_view);
+}
+
+static void
+ggv_postscript_view_destroy_dialogs(GgvPostScriptView *ps_view)
+{
+	GList *dialog_node = ps_view->priv->dialogs;
+	GtkWidget *dialog;
+
+	while(dialog_node) {
+		dialog = GTK_WIDGET(dialog_node->data);
+		dialog_node = dialog_node->next;
+		gtk_widget_destroy(dialog);
+	}
+}
+
 static void
 set_file_items_sensitivity(GgvPostScriptView *ps_view, gboolean sens)
 {
@@ -314,10 +345,12 @@ ggv_postscript_view_get_page_count(GgvDocument *doc)
 	return gtk_gs_get_page_count(GTK_GS(ps_view->priv->gs));
 }
 
-static void
+static gboolean
 load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 {
-	gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
+	gboolean rv;
+
+	rv = gtk_gs_load(GTK_GS(ps_view->priv->gs), fname);
 	notify_title_change(ps_view);
 	notify_page_count_change(ps_view);
 	notify_orientation_change(ps_view);
@@ -326,6 +359,7 @@ load_ps(GgvPostScriptView *ps_view, const gchar *fname)
 	ggv_sidebar_update_coordinates(ps_view->priv->sidebar, 0.0, 0.0);
 	set_file_items_sensitivity(ps_view, ggv_document_get_page_count(GGV_DOCUMENT(ps_view)) > 0);
 	notify_status_change(ps_view);
+	return rv;
 }
 
 static void
@@ -404,7 +438,10 @@ load_ps_from_stream (BonoboPersistStream *ps,
 	} while (len_read > 0);
 
 	fclose(tmpfile);
-	load_ps(ps_view, ps_view->priv->tmp_name);
+	if(!load_ps(ps_view, ps_view->priv->tmp_name)) {
+		CORBA_exception_set(ev,	CORBA_USER_EXCEPTION,
+							ex_GNOME_GGV_DocumentError, NULL);
+	}
 	return;
 
  exit_clean:
@@ -523,6 +560,7 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 	GgvPostScriptView *ps_view;
 	gchar *local_file = NULL, *uri_str;
 	FILE *tmpfile;
+	gint rv;
 
 	g_return_val_if_fail (data != NULL, -1);
 	g_return_val_if_fail (GGV_IS_POSTSCRIPT_VIEW (data), -1);
@@ -573,15 +611,24 @@ load_ps_from_file (BonoboPersistFile *pf, const CORBA_char *filename,
 		if(result != GNOME_VFS_ERROR_EOF)
 			return -1;
 	}
+	rv = 0;
 	if(ps_view->priv->tmp_name) {
-		load_ps(ps_view, ps_view->priv->tmp_name);
+		if(!load_ps(ps_view, ps_view->priv->tmp_name)) {
+			CORBA_exception_set(ev,	CORBA_USER_EXCEPTION,
+								ex_GNOME_GGV_DocumentError, NULL);
+			rv = -1;
+		}
 	}
 	else if(local_file) {
-		load_ps(ps_view, local_file);
+		if(!load_ps(ps_view, local_file)) {
+			CORBA_exception_set(ev,	CORBA_USER_EXCEPTION,
+								ex_GNOME_GGV_DocumentError, NULL);
+			rv = -1;
+		}
 		g_free(local_file);
 	}
 
-	return 0;
+	return rv;
 }
 
 static void
@@ -764,6 +811,7 @@ ggv_postscript_view_print(GgvPostScriptView *ps_view, gchar *ps)
 									 GTK_BUTTONS_OK,
 									 _("Unable to execute print command:\n%s"),
 									 ggv_print_cmd);
+		add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
 		if(ps_view->priv->owner != NULL) {
 			bonobo_control_set_transient_for(ps_view->priv->owner,
 											 GTK_WINDOW(dlg), NULL);
@@ -823,6 +871,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 									 GTK_BUTTONS_YES_NO,
 									 _("No pages have been marked.\n"
 									   "Do you want to save the whole document?"));
+		add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
 		if(ps_view->priv->owner != NULL) {
 			bonobo_control_set_transient_for(ps_view->priv->owner,
 											 GTK_WINDOW(dlg), NULL);
@@ -854,6 +903,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 											  GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
 											  GTK_STOCK_SAVE, GTK_RESPONSE_OK,
 											  NULL);
+		add_dialog_to_view(ps_view, GTK_WIDGET(chooser));
 		if(ps_view->priv->owner != NULL) {
 			bonobo_control_set_transient_for(ps_view->priv->owner,
 											 GTK_WINDOW(chooser), NULL);
@@ -887,6 +937,7 @@ verb_FileSaveMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 											 GTK_BUTTONS_YES_NO,
 											 _("A file with this name already exists.\n"
 											   "Do you want to overwrite it?"));
+				add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
 				if(ps_view->priv->owner != NULL) {
 					bonobo_control_set_transient_for(ps_view->priv->owner,
 													 GTK_WINDOW(dlg), NULL);
@@ -931,11 +982,12 @@ verb_FilePrintMarked(BonoboUIComponent *uic, gpointer data, const char *cname)
 									 GTK_BUTTONS_YES_NO,
 									 _("No pages have been marked.\n"
 									   "Do you want to print the whole document?"));
+		add_dialog_to_view(ps_view, GTK_WIDGET(dlg));
 		if(ps_view->priv->owner != NULL) {
 			bonobo_control_set_transient_for(ps_view->priv->owner,
 											 GTK_WINDOW(dlg), NULL);
 			gtk_window_set_destroy_with_parent(GTK_WINDOW(dlg), TRUE);
-		}			
+		}
 		gtk_widget_show(dlg);
 		switch(gtk_dialog_run(GTK_DIALOG(dlg))) {
 		case GTK_RESPONSE_NO:
@@ -1668,6 +1720,8 @@ ggv_postscript_view_destroy(BonoboObject *object)
 	}
 	ps_view_clean_tmp_file(ps_view);
 
+	ggv_postscript_view_destroy_dialogs(ps_view);
+
 	if (BONOBO_OBJECT_CLASS (ggv_postscript_view_parent_class)->destroy)
 		BONOBO_OBJECT_CLASS(ggv_postscript_view_parent_class)->destroy(object);
 }
