@@ -60,9 +60,7 @@
 #include "ps.h"
 #include "gsio.h"
 
-#ifdef BSD4_2
-#define memset(a,b,c) bzero(a,c)
-#endif
+#include <glib.h>
 
 /* length calculates string length at compile time */
 /* can only be used with character constants */
@@ -86,6 +84,21 @@ static int  blank();
 #endif
 
 
+static struct page *
+pages_new(struct page *pages, int current, int maxpages)
+{
+        struct page *oldpages = pages;
+        if(!oldpages)
+                pages = g_new0(struct page, maxpages);
+        else
+                pages = g_renew(struct page, oldpages, maxpages);
+        for(; current < maxpages; current++) {
+                memset(&(pages[current]), 0x00, sizeof(struct page));
+                pages[current].orientation = GTK_GS_ORIENTATION_NONE;
+        }
+        return pages;
+}
+
 /*
  *	psscan -- scan the PostScript file for document structuring comments.
  *
@@ -202,7 +215,7 @@ static int  blank();
 
 #include <gtkgs.h>
 
-struct document *psscan (FILE *file, int respect_eof)
+struct document *psscan (FILE *file, int respect_eof, const gchar *fname)
 {
     struct document *doc;
     int bb_set = NONE;
@@ -258,6 +271,8 @@ struct document *psscan (FILE *file, int respect_eof)
      * parts of this or add a ^D at the beginning. */
     if (iscomment (line,"%!PS") || iscomment (line, "\004%!PS")) {
 	doc = g_new0 (struct document, 1);
+        doc->default_page_orientation = GTK_GS_ORIENTATION_NONE;
+        doc->orientation = GTK_GS_ORIENTATION_NONE;
 
         /* ignore possible leading ^D */
 	if (*line == '\004') {
@@ -267,7 +282,8 @@ struct document *psscan (FILE *file, int respect_eof)
 
 /* Jake Hamby patch 18/3/98 */
 
-	sscanf(line, "%*s %s", text);
+        text[0] = '\0';
+	sscanf(line, "%*s %256s", text);
 	/*doc->epsf = iscomment(text, "EPSF-");*/
 	doc->epsf = iscomment(text, "EPSF"); /* Hamby - This line changed */
 	doc->beginheader = position;
@@ -286,10 +302,25 @@ struct document *psscan (FILE *file, int respect_eof)
 
 	 In a way, this makes sense, a program PostScript does not need
 	 the !PS at the beginning.
-
       */
-	doc = g_new0 (struct document, 1);
-	return(doc);
+            /* use a test command to determine if ghostscript can
+               understand this document! */
+            gchar *test_cmd;
+
+            test_cmd = g_strdup_printf
+                    ("%s -dNOPAUSE -dBATCH -sDEVICE=nullpage %s "
+                     "1>/dev/null 2>/dev/null",
+                     gtk_gs_defaults_get_interpreter_cmd(),
+                     fname);
+            if(system(test_cmd) != 0) {
+                    g_free(test_cmd);
+                    return NULL;
+            }
+            g_free(test_cmd);
+            doc = g_new0 (struct document, 1);
+            doc->default_page_orientation = GTK_GS_ORIENTATION_NONE;
+            doc->orientation = GTK_GS_ORIENTATION_NONE;
+            return(doc);
     }
 
     preread = 0;
@@ -308,7 +339,7 @@ struct document *psscan (FILE *file, int respect_eof)
 	} else if (doc->date == NULL && iscomment(line+2, "CreationDate:")) {
 	    doc->date = gettextline(line+length("%%CreationDate:"));
 	} else if (bb_set == NONE && iscomment(line+2, "BoundingBox:")) {
-	    sscanf(line+length("%%BoundingBox:"), "%s", text);
+	    sscanf(line+length("%%BoundingBox:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		bb_set = ATEND;
 	    } else {
@@ -340,7 +371,7 @@ struct document *psscan (FILE *file, int respect_eof)
 	    }
 	} else if (orientation_set == NONE &&
 		   iscomment(line+2, "Orientation:")) {
-	    sscanf(line+length("%%Orientation:"), "%s", text);
+	    sscanf(line+length("%%Orientation:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		orientation_set = ATEND;
 	    } else if (strcmp(text, "Portrait") == 0) {
@@ -355,7 +386,7 @@ struct document *psscan (FILE *file, int respect_eof)
                     orientation_set = 1;
             }
 	} else if (page_order_set == NONE && iscomment(line+2, "PageOrder:")) {
-	    sscanf(line+length("%%PageOrder:"), "%s", text);
+	    sscanf(line+length("%%PageOrder:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		page_order_set = ATEND;
 	    } else if (strcmp(text, "Ascend") == 0) {
@@ -369,7 +400,7 @@ struct document *psscan (FILE *file, int respect_eof)
 		page_order_set = 1;
 	    }
 	} else if (pages_set == NONE && iscomment(line+2, "Pages:")) {
-	    sscanf(line+length("%%Pages:"), "%s", text);
+	    sscanf(line+length("%%Pages:"), "%256s", text);
 	    if (strcmp(text, "(atend)") == 0) {
 		pages_set = ATEND;
 	    } else {
@@ -390,7 +421,7 @@ struct document *psscan (FILE *file, int respect_eof)
 			}
 		    case 1:
 			if (maxpages > 0)
-			    doc->pages = g_new0 (struct page, maxpages);
+			    doc->pages = pages_new(NULL, 0, maxpages);
 		}
 	    }
 	} else if (doc->numsizes == NONE &&
@@ -568,7 +599,7 @@ struct document *psscan (FILE *file, int respect_eof)
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%s", text);
+		sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -695,7 +726,7 @@ struct document *psscan (FILE *file, int respect_eof)
 		/* Do nothing */
 	    } else if (doc->default_page_orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%s", text);
+		sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->default_page_orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -799,7 +830,7 @@ newpage:
     while (DSCcomment(line) && iscomment(line+2, "Page:")) {
 	if (maxpages == 0) {
 	    maxpages = 1;
-	    doc->pages = g_new0 (struct page, maxpages);
+	    doc->pages = pages_new(NULL, 0, maxpages);
 	}
 	label = get_next_text(line+length("%%Page:"), &next_char);
 	if (sscanf(next_char, "%d", &thispage) != 1) thispage = 0;
@@ -814,9 +845,8 @@ newpage:
 	nextpage++;
 	if (doc->numpages == maxpages) {
 	    maxpages++;
-	    doc->pages = g_renew (struct page, doc->pages, maxpages);
+	    doc->pages = pages_new (doc->pages, maxpages - 1, maxpages);
 	}
-	memset(&(doc->pages[doc->numpages]), 0, sizeof(struct page));
 	page_bb_set = NONE;
 	doc->pages[doc->numpages].label = label;
 	if (beginsection) {
@@ -837,7 +867,7 @@ continuepage:
 		/* Do nothing */
 	    } else if (doc->pages[doc->numpages].orientation == NONE &&
 		iscomment(line+2, "PageOrientation:")) {
-		sscanf(line+length("%%PageOrientation:"), "%s", text);
+		sscanf(line+length("%%PageOrientation:"), "%256s", text);
 		if (strcmp(text, "Portrait") == 0) {
 		    doc->pages[doc->numpages].orientation = GTK_GS_ORIENTATION_PORTRAIT;
 		} else if (strcmp(text, "Landscape") == 0) {
@@ -872,7 +902,7 @@ continuepage:
 		g_free(cp);
 	    } else if ((page_bb_set == NONE || page_bb_set == ATEND) &&
 		       iscomment(line+2, "PageBoundingBox:")) {
-		sscanf(line+length("%%PageBoundingBox:"), "%s", text);
+		sscanf(line+length("%%PageBoundingBox:"), "%256s", text);
 		if (strcmp(text, "(atend)") == 0) {
 		    page_bb_set = ATEND;
 		} else {
@@ -986,7 +1016,7 @@ continuepage:
 	    }
 	} else if (orientation_set == ATEND &&
 		   iscomment(line+2, "Orientation:")) {
-	    sscanf(line+length("%%Orientation:"), "%s", text);
+	    sscanf(line+length("%%Orientation:"), "%256s", text);
 	    if (strcmp(text, "Portrait") == 0) {
 		doc->orientation = GTK_GS_ORIENTATION_PORTRAIT;
 	    } else if (strcmp(text, "Landscape") == 0) {
@@ -996,7 +1026,7 @@ continuepage:
                     doc->orientation = GTK_GS_ORIENTATION_SEASCAPE;
             }
 	} else if (page_order_set == ATEND && iscomment(line+2, "PageOrder:")) {
-	    sscanf(line+length("%%PageOrder:"), "%s", text);
+	    sscanf(line+length("%%PageOrder:"), "%256s", text);
 	    if (strcmp(text, "Ascend") == 0) {
 		doc->pageorder = ASCEND;
 	    } else if (strcmp(text, "Descend") == 0) {
@@ -1278,7 +1308,7 @@ readline(lineP, size, fp, positionP, line_lenP)
    else if IS_BEGIN("Data:")      {
 	text[0] = '\0';
 	strcpy(save, line+7);
-	if (sscanf(line+length("%%BeginData:"), "%d %*s %s", &num, text) >= 1) {
+	if (sscanf(line+length("%%BeginData:"), "%d %*s %256s", &num, text) >= 1) {
 	    if (strcmp(text, "Lines") == 0) {
 		for (i=0; i < num; i++) {
 		    cp = fgets(line, size, fp);
@@ -1402,7 +1432,7 @@ readline(line, size, fp, position, line_len)
     } else if (iscomment(line+7, "Data:")) {
 	text[0] = '\0';
 	strcpy(save, line+7);
-	if (sscanf(line+length("%%BeginData:"), "%d %*s %s", &num, text) >= 1) {
+	if (sscanf(line+length("%%BeginData:"), "%d %*s %256s", &num, text) >= 1) {
 	    if (strcmp(text, "Lines") == 0) {
 		for (i=0; i < num; i++) {
 		    cp = fgets(line, size, fp);
@@ -1477,7 +1507,7 @@ pscopy(from, to, begin, end)
                 } else if (iscomment(line+7, "Data:")) {
                         text[0] = '\0';
                         if (sscanf(line+length("%%BeginData:"),
-                                   "%d %*s %s", &num, text) >= 1) {
+                                   "%d %*s %256s", &num, text) >= 1) {
                                 if (strcmp(text, "Lines") == 0) {
                                         for (i=0; i < num; i++) {
                                                 fgets(line, sizeof(line), from);
@@ -1546,7 +1576,7 @@ pscopyuntil(FILE *from, GtkGSDocSink *to, long begin, long end,
                 } else if (iscomment(line+7, "Data:")) {
                         text[0] = '\0';
                         if (sscanf(line+length("%%BeginData:"),
-                                   "%d %*s %s", &num, text) >= 1) {
+                                   "%d %*s %256s", &num, text) >= 1) {
                                 if (strcmp(text, "Lines") == 0) {
                                         for (i=0; i < num; i++) {
                                                 fgets(line, sizeof line, from);
@@ -1624,7 +1654,7 @@ void pscopydoc(GtkGSDocSink *dest,
                         g_free(comment);
                         continue;
                 }
-                sscanf(comment+length("%%Pages:"), "%s", text);
+                sscanf(comment+length("%%Pages:"), "%256s", text);
                 if (strcmp(text, "(atend)") == 0) {
                         gtk_gs_doc_sink_write(dest, comment, strlen(comment));
                         pages_atend = TRUE;
